<?php


namespace FiloBlu\CustomerAccountAge\Model;

class Core
{
    protected $_resource;
    protected $connection = false;
    protected $setup;

    protected $attribute_code = false;
    protected $attribute_id = false;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $resource,
        \FiloBlu\CustomerAccountAge\Setup\Patch\Data\AddAgeDaysCustomerAttribute $setup
    )
    {
        $this->_resource = $resource;
        $this->setup = $setup;
    }

    public function executeUpdate(){

        $this->attribute_code = $this->setup->getAttributeCode();
        $sql = "SELECT attribute_id FROM eav_attribute WHERE attribute_code = '{$this->attribute_code}' AND entity_type_id = 1;";
        $this->attribute_id = $this->_resource->getConnection()->fetchOne($sql);
        if(!$this->attribute_id){
            return false;
        }

        $ages = $this->getCustomersWithAgeToBeUpdated();
        if(count($ages) == 0){
            return false;
        }

        // Building a single update query
        $update_queries = array();
        foreach ($ages as $customer_age_data) {
            $update_queries[] = " ({$this->attribute_id}, {$customer_age_data['entity_id']}, {$customer_age_data[$this->attribute_code]})\n ";
        }

        // Here we probably have a lot of updates
        // Splitting the array into 5000 pieces chunks to preserve performance

        $chunks = array_chunk($update_queries,5000,true);

        foreach ($chunks as $chunk) {

            $update_queries_string = implode(",",$chunk);
            $sql = " INSERT INTO customer_entity_varchar (attribute_id, entity_id, value) VALUES \n";
            $sql .= $update_queries_string;
            $sql .= " ON DUPLICATE KEY UPDATE value=VALUES(value); ";

            $this->_resource->getConnection()->query($sql);

        }

        return true;
    }

    protected function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection('core_write');
        }
        return $this->connection;
    }

    protected function getCustomersWithAgeToBeUpdated(){

        $sql = "SELECT CE.entity_id
                ,CE.store_id
                ,CE.website_id
                ,CE.email
                ,CE.group_id
                ,CE.created_at
                ,DATEDIFF(CAST(NOW() AS DATE), CAST(CE.created_at AS DATE)) AS '{$this->attribute_code}'
                ,CEV.value AS 'current_saved_{$this->attribute_code}'
                FROM customer_entity CE
                LEFT JOIN customer_entity_varchar CEV ON CEV.entity_id = CE.entity_id AND CEV.attribute_id = {$this->attribute_id}
                WHERE CEV.value != DATEDIFF(CAST(NOW() AS DATE), CAST(CE.created_at AS DATE)) OR CEV.value IS NULL;";

        $result = $this->getConnection()->fetchAll($sql);
        return $result;
    }
}